@@ -19,6 +19,6 @@
 #ifndef _DEVICE_BLKIO_H_
 #define _DEVICE_BLKIO_H_
 
-extern vm_offset_t block_io_mmap(void);
+extern int block_io_mmap(dev_t dev, vm_offset_t off, int prot);
 
 #endif /* _DEVICE_BLKIO_H_ */
