@@ -89,9 +89,8 @@ kern_return_t task_create(
 #endif
 
 	new_task = (task_t) kmem_cache_alloc(&task_cache);
-	if (new_task == TASK_NULL) {
-		panic("task_create: no memory for task structure");
-	}
+	if (new_task == TASK_NULL)
+		return KERN_RESOURCE_SHORTAGE;
 
 	/* one ref for just being alive; one for our caller */
 	new_task->ref_count = 2;
@@ -378,7 +377,7 @@ kern_return_t task_terminate(
                 task_unlock(task);
                 thread_force_terminate(thread);
                 thread_deallocate(thread);
-                thread_block((void (*)()) 0);
+                thread_block(thread_no_continuation);
                 task_lock(task);
         }
         task_unlock(task);
@@ -784,7 +783,8 @@ kern_return_t task_info(
 				= task->total_system_time.seconds;
 		basic_info->system_time.microseconds
 				= task->total_system_time.microseconds;
-		basic_info->creation_time = task->creation_time;
+		read_time_stamp(&task->creation_time,
+				&basic_info->creation_time);
 		task_unlock(task);
 
 		if (*task_info_count > TASK_BASIC_INFO_COUNT)
@@ -893,7 +893,7 @@ task_assign(
 		task->assign_active = TRUE;
 		assert_wait((event_t)&task->assign_active, TRUE);
 		task_unlock(task);
-		thread_block((void (*)()) 0);
+		thread_block(thread_no_continuation);
 		task_lock(task);
 	}
 
@@ -1064,6 +1064,9 @@ kern_return_t task_get_assignment(
 	task_t		task,
 	processor_set_t	*pset)
 {
+	if (task == TASK_NULL)
+		return KERN_INVALID_ARGUMENT;
+
 	if (!task->active)
 		return KERN_FAILURE;
 
